"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.completion = exports.commandNames = exports.cliOptionsTypes = void 0;
exports.rcOptionsTypes = rcOptionsTypes;
exports.help = help;
exports.handler = handler;
const cli_utils_1 = require("@pnpm/cli-utils");
const common_cli_options_help_1 = require("@pnpm/common-cli-options-help");
const config_1 = require("@pnpm/config");
const error_1 = require("@pnpm/error");
const get_context_1 = require("@pnpm/get-context");
const workspace_find_packages_1 = require("@pnpm/workspace.find-packages");
const manifest_utils_1 = require("@pnpm/manifest-utils");
const store_connection_manager_1 = require("@pnpm/store-connection-manager");
const core_1 = require("@pnpm/core");
const pick_1 = __importDefault(require("ramda/src/pick"));
const without_1 = __importDefault(require("ramda/src/without"));
const render_help_1 = __importDefault(require("render-help"));
const getSaveType_1 = require("./getSaveType");
const recursive_1 = require("./recursive");
class RemoveMissingDepsError extends error_1.PnpmError {
    constructor(opts) {
        let message = 'Cannot remove ';
        message += `${opts.nonMatchedDependencies.map(dep => `'${dep}'`).join(', ')}: `;
        if (opts.availableDependencies.length > 0) {
            message += `no such ${opts.nonMatchedDependencies.length > 1 ? 'dependencies' : 'dependency'} `;
            message += `found${opts.targetDependenciesField ? ` in '${opts.targetDependenciesField}'` : ''}`;
            const hint = `Available dependencies: ${opts.availableDependencies.join(', ')}`;
            super('CANNOT_REMOVE_MISSING_DEPS', message, { hint });
            return;
        }
        message += opts.targetDependenciesField
            ? `project has no '${opts.targetDependenciesField}'`
            : 'project has no dependencies of any kind';
        super('CANNOT_REMOVE_MISSING_DEPS', message);
    }
}
function rcOptionsTypes() {
    return (0, pick_1.default)([
        'cache-dir',
        'global-dir',
        'global-pnpmfile',
        'global',
        'lockfile-dir',
        'lockfile-directory',
        'lockfile-only',
        'lockfile',
        'node-linker',
        'package-import-method',
        'pnpmfile',
        'reporter',
        'save-dev',
        'save-optional',
        'save-prod',
        'shared-workspace-lockfile',
        'store-dir',
        'strict-peer-dependencies',
        'virtual-store-dir',
    ], config_1.types);
}
const cliOptionsTypes = () => ({
    ...rcOptionsTypes(),
    ...(0, pick_1.default)(['force'], config_1.types),
    recursive: Boolean,
});
exports.cliOptionsTypes = cliOptionsTypes;
function help() {
    return (0, render_help_1.default)({
        aliases: ['rm', 'uninstall', 'un'],
        description: 'Removes packages from `node_modules` and from the project\'s `package.json`.',
        descriptionLists: [
            {
                title: 'Options',
                list: [
                    {
                        description: 'Remove from every package found in subdirectories \
or from every workspace package, when executed inside a workspace. \
For options that may be used with `-r`, see "pnpm help recursive"',
                        name: '--recursive',
                        shortAlias: '-r',
                    },
                    {
                        description: 'Remove the dependency only from "devDependencies"',
                        name: '--save-dev',
                        shortAlias: '-D',
                    },
                    {
                        description: 'Remove the dependency only from "optionalDependencies"',
                        name: '--save-optional',
                        shortAlias: '-O',
                    },
                    {
                        description: 'Remove the dependency only from "dependencies"',
                        name: '--save-prod',
                        shortAlias: '-P',
                    },
                    common_cli_options_help_1.OPTIONS.globalDir,
                    ...common_cli_options_help_1.UNIVERSAL_OPTIONS,
                ],
            },
            common_cli_options_help_1.FILTERING,
        ],
        url: (0, cli_utils_1.docsUrl)('remove'),
        usages: ['pnpm remove <pkg>[@<version>]...'],
    });
}
// Unlike npm, pnpm does not treat "r" as an alias of "remove".
// This way we avoid the confusion about whether "pnpm r" means remove, run, or recursive.
exports.commandNames = ['remove', 'uninstall', 'rm', 'un', 'uni'];
const completion = async (cliOpts) => {
    return (0, cli_utils_1.readDepNameCompletions)(cliOpts.dir);
};
exports.completion = completion;
async function handler(opts, params) {
    if (params.length === 0)
        throw new error_1.PnpmError('MUST_REMOVE_SOMETHING', 'At least one dependency name should be specified for removal');
    const include = {
        dependencies: opts.production !== false,
        devDependencies: opts.dev !== false,
        optionalDependencies: opts.optional !== false,
    };
    if (opts.recursive && (opts.allProjects != null) && (opts.selectedProjectsGraph != null) && opts.workspaceDir) {
        await (0, recursive_1.recursive)(opts.allProjects, params, {
            ...opts,
            allProjectsGraph: opts.allProjectsGraph,
            include,
            selectedProjectsGraph: opts.selectedProjectsGraph,
            workspaceDir: opts.workspaceDir,
        }, 'remove');
        return;
    }
    const store = await (0, store_connection_manager_1.createOrConnectStoreController)(opts);
    const removeOpts = Object.assign(opts, {
        ...(0, config_1.getOptionsFromRootManifest)(opts.rootProjectManifestDir, opts.rootProjectManifest ?? {}),
        linkWorkspacePackagesDepth: opts.linkWorkspacePackages === 'deep' ? Infinity : opts.linkWorkspacePackages ? 0 : -1,
        storeController: store.ctrl,
        storeDir: store.dir,
        include,
    });
    // @ts-expect-error
    removeOpts['workspacePackages'] = opts.workspaceDir
        ? (0, get_context_1.arrayOfWorkspacePackagesToMap)(await (0, workspace_find_packages_1.findWorkspacePackages)(opts.workspaceDir, { ...opts, patterns: opts.workspacePackagePatterns }))
        : undefined;
    const targetDependenciesField = (0, getSaveType_1.getSaveType)(opts);
    const { manifest: currentManifest, writeProjectManifest, } = await (0, cli_utils_1.readProjectManifest)(opts.dir, opts);
    const availableDependencies = Object.keys(targetDependenciesField === undefined
        ? (0, manifest_utils_1.getAllDependenciesFromManifest)(currentManifest)
        : currentManifest[targetDependenciesField] ?? {});
    const nonMatchedDependencies = (0, without_1.default)(availableDependencies, params);
    if (nonMatchedDependencies.length !== 0) {
        throw new RemoveMissingDepsError({
            availableDependencies,
            nonMatchedDependencies,
            targetDependenciesField,
        });
    }
    const mutationResult = await (0, core_1.mutateModulesInSingleProject)({
        binsDir: opts.bin,
        dependencyNames: params,
        manifest: currentManifest,
        mutation: 'uninstallSome',
        rootDir: opts.dir,
        targetDependenciesField,
    }, removeOpts);
    await writeProjectManifest(mutationResult.manifest);
}
//# sourceMappingURL=remove.js.map